from . import config
from . import decoder
from datetime import datetime
import RPi.GPIO as GPIO

def setup(**options):
	return config.setup(**options)

def listen(callback):
	try:

		if(not config.opts['signalPin']):
			raise Exception('You should set your signal PIN by using setup() function before listening')
		GPIO.setmode(GPIO.BOARD)
		GPIO.setup(config.opts['signalPin'], GPIO.IN)

		dec = decoder.Decoder()
		while True:
			defaultCycles = 0
			value = config.opts['defaultInput']

			while(value == config.opts['defaultInput']):
				value = (GPIO.input(config.opts['signalPin']))

			lastTime = datetime.now()
			previousValue = value

			while True:
				if(previousValue != value):
					diff = datetime.now()-lastTime
					dec.record((datetime.now()-lastTime).microseconds)
					lastTime = datetime.now()

				previousValue = value

				if(value == config.opts['defaultInput']):
					defaultCycles = defaultCycles+1
				
				if(defaultCycles > config.opts['defaultCycles']):
					result = dec.decode()
					dec.reset()
					callback(result)
					break

				value = GPIO.input(config.opts['signalPin'])
	finally:
		GPIO.cleanup()



from distutils.core import setup
setup(
  name = 'raspir',
  packages = ['raspir'],
  version = '1.1',
  description = 'A library to detect IR signals from receivers on Raspberry Pi',
  author = 'Pavlo Kochylo',
  author_email = 'kochylo.paolo@gmail.com',
  url = 'https://github.com/SyncroIT/Raspir',
  download_url = 'https://github.com/SyncroIT/Raspir/archive/1.1.tar.gz',
  keywords = ['ir', 'raspberry', 'infrared', 'decoder'],
  classifiers = [],
)



import math
import time
from . import config

class Decoder:
	recorded = []
	lastDecode = False
	lastDecodeTime = time.time()

	def record(self, value):
		self.recorded.append(value)

	def decode(self):
		ms = time.time()*1000
		if(self.lastDecodeTime and (ms-self.lastDecodeTime) > config.opts['floodInterval']):
			binary = []
			for (timing) in self.recorded:
				binary.append(int(timing > config.opts['divider']))
			self.lastDecode = int("".join(map(str, binary)),2)%10000

		self.lastDecodeTime = ms
		return self.lastDecode

	def reset(self):
		self.recorded = []


opts = {
	#This is the default input that Raspberry receives by your IR receiver while there's no IR light pointed at. In my case it's 1.
	'defaultInput': 1, 

	#This is the amount of inputs equal to defaultInput to achieve to make Raspberry know that IR signal is ended. 
	'defaultCycles': 10000,

	#It's the GPIO pin where there's connected your IR receiver. NB: GPIO.BOARD numeration used.
	'signalPin': False,

	#It should be a number equal to 1/10 of defaultCycles. It's not random and if changed it will also make change the result of decoding operation.
	#It's used to mark a number as true or not (x < divider)
	'divider': 1000,

	#It's the amount of milliseconds to distinguish a new signal from the last one
	#Every signal sent after another one, in this timing will be treated as a copy of the last one.
	'floodInterval': 250
}

def setup(**options):
	for(name) in options:
		opts[name] = options[name]

